@@ -30,7 +30,7 @@ from robotframework_debug_adapter.protocols import (
     IBusyWait,
     IEvaluationInfo,
 )
-from typing import Optional, List, Iterable, Union, Any, Dict, FrozenSet
+from typing import Optional, List, Iterable, Union, Any, Dict, FrozenSet, Tuple
 from robocorp_ls_core.basic import implements
 from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
     StackFrame,
@@ -384,9 +384,12 @@ class _StackInfo(object):
         return lst
 
 
-_StepEntry = namedtuple("_StepEntry", "name, lineno, source, args, variables")
-_SuiteEntry = namedtuple("_SuiteEntry", "name, source")
-_TestEntry = namedtuple("_TestEntry", "name, source, lineno")
+_StepEntry = namedtuple(
+    "_StepEntry", "name, lineno, source, args, variables, entry_type"
+)
+_SuiteEntry = namedtuple("_SuiteEntry", "name, source, entry_type")
+_TestEntry = namedtuple("_TestEntry", "name, source, lineno, entry_type")
+_LogEntry = namedtuple("_LogEntry", "name, source, lineno, entry_type")
 
 
 class InvalidFrameIdError(Exception):
@@ -542,6 +545,19 @@ class _RobotDebuggerImpl(object):
         self._evaluations = []
         self._skip_breakpoints = 0
 
+        self._exc_name = None
+        self._exc_description = None
+
+        def is_true_in_env(key):
+            return os.getenv(key, "0").lower() in (
+                "1",
+                "True",
+                "true",
+            )
+
+        self.break_on_log_failure = is_true_in_env("RFLS_BREAK_ON_FAILURE")
+        self.break_on_log_error = is_true_in_env("RFLS_BREAK_ON_ERROR")
+
     def write_message(self, msg):
         log.critical(
             "Message: %s not sent!\nExpected _RobotDebuggerImpl.write_message to be replaced to the actual implementation!",
@@ -553,6 +569,14 @@ class _RobotDebuggerImpl(object):
     def stop_reason(self) -> ReasonEnum:
         return self._reason
 
+    @property
+    def exc_name(self) -> Optional[str]:
+        return self._exc_name
+
+    @property
+    def exc_description(self) -> Optional[str]:
+        return self._exc_description
+
     def _get_stack_info_from_frame_id(self, frame_id) -> Optional[_StackInfo]:
         thread_id = self._frame_id_to_tid.get(frame_id)
         if thread_id is not None:
@@ -630,6 +654,14 @@ class _RobotDebuggerImpl(object):
                     name = "TestCase: %s" % (entry.name,)
                     filename = self._get_filename(entry, "TestCase")
 
+                    frame_id = stack_info.add_test_entry_stack(
+                        name, filename, entry.lineno
+                    )
+
+                elif entry.__class__ == _LogEntry:
+                    name = "Log (%s)" % (entry.name,)
+                    filename = self._get_filename(entry, "Log")
+
                     frame_id = stack_info.add_test_entry_stack(
                         name, filename, entry.lineno
                     )
@@ -657,7 +689,15 @@ class _RobotDebuggerImpl(object):
     def wait_suspended(self, reason: ReasonEnum) -> None:
         thread_id = self.get_current_thread_id()
 
-        log.info("wait_suspended", reason)
+        if self._exc_name or self._exc_description:
+            log.info(
+                "wait_suspended. Reason: %s. Exc name: %s. Exc description: %s",
+                reason,
+                self._exc_name,
+                self._exc_description,
+            )
+        else:
+            log.info("wait_suspended. Reason: %s", reason)
         self._create_stack_info(thread_id)
         try:
             self._run_state = STATE_PAUSED
@@ -680,6 +720,11 @@ class _RobotDebuggerImpl(object):
 
             if self._step_cmd == StepEnum.STEP_NEXT:
                 self._stop_on_stack_len = len(self._stack_ctx_entries_deque)
+                if self._stop_on_stack_len:
+                    if self._is_control_step(
+                        self._stack_ctx_entries_deque[-1].entry_type
+                    ):
+                        self._stop_on_stack_len += 1
 
             elif self._step_cmd == StepEnum.STEP_OUT:
                 self._stop_on_stack_len = len(self._stack_ctx_entries_deque) - 1
@@ -759,11 +804,12 @@ class _RobotDebuggerImpl(object):
         source = attributes["source"]
         name = attributes["kwname"]
         args = attributes["args"]
+        entry_type = attributes.get("type", "KEYWORD")
         if attributes.get("status") == "NOT RUN":
             return
         if not args:
             args = []
-        self._before_run_step(ctx, name, lineno, source, args)
+        self._before_run_step(ctx, name, entry_type, lineno, source, args)
 
     # 4.0 versions where the lineno is available on the V2 listener
     def end_keyword_v2(self, name, attributes):
@@ -801,7 +847,8 @@ class _RobotDebuggerImpl(object):
             args = control_flow_stmt.args
         except AttributeError:
             args = []
-        self._before_run_step(ctx, name, lineno, source, args)
+        entry_type = "KEYWORD"
+        self._before_run_step(ctx, name, entry_type, lineno, source, args)
 
     # 3.x versions where the lineno is NOT available on the V2 listener
     def after_control_flow_stmt(self, control_flow_stmt, ctx, *args, **kwargs):
@@ -825,7 +872,8 @@ class _RobotDebuggerImpl(object):
         except AttributeError:
             args = []
         ctx = runner._context
-        self._before_run_step(ctx, name, lineno, source, args)
+        entry_type = "KEYWORD"
+        self._before_run_step(ctx, name, entry_type, lineno, source, args)
 
     def after_keyword_runner(self, runner, step, *args, **kwargs):
         self._after_run_step()
@@ -848,13 +896,33 @@ class _RobotDebuggerImpl(object):
         except AttributeError:
             args = []
         ctx = step_runner._context
-        self._before_run_step(ctx, name, lineno, source, args)
+        entry_type = "KEYWORD"
+        self._before_run_step(ctx, name, entry_type, lineno, source, args)
 
     # 3.x versions where the lineno is NOT available on the V2 listener
     def after_run_step(self, step_runner, step, name=None):
         self._after_run_step()
 
-    def _before_run_step(self, ctx, name, lineno, source, args):
+    def _is_control_step(self, entry_type):
+        return entry_type in (
+            "FOR",
+            "FOR ITERATION",
+            "ITERATION",
+            "IF",
+            "ELSE",
+            "ELSE IF",
+            "TRY",
+            "EXCEPT",
+            "WHILE",
+        )
+
+    def _before_run_step(self, ctx, name, entry_type, lineno, source, args):
+        if not name:
+            name = entry_type
+
+        if self._is_control_step(entry_type):
+            self._stop_on_stack_len += 1
+
         if source is None or lineno is None:
             # RunKeywordIf doesn't have a source, so, just show the caller source.
             for entry in reversed(self._stack_ctx_entries_deque):
@@ -871,7 +939,7 @@ class _RobotDebuggerImpl(object):
             if os.path.exists(robot_init):
                 source = robot_init
         self._stack_ctx_entries_deque.append(
-            _StepEntry(name, lineno, source, args, ctx.variables.current)
+            _StepEntry(name, lineno, source, args, ctx.variables.current, entry_type)
         )
         if self._skip_breakpoints:
             return
@@ -967,46 +1035,111 @@ class _RobotDebuggerImpl(object):
             self.wait_suspended(stop_reason)
 
     def _after_run_step(self):
-        self._stack_ctx_entries_deque.pop()
+        entry = self._stack_ctx_entries_deque.pop()
+        if self._is_control_step(entry.entry_type):
+            self._stop_on_stack_len -= 1
 
     def start_suite(self, data, result):
-        self._stack_ctx_entries_deque.append(_SuiteEntry(data.name, data.source))
+        self._stack_ctx_entries_deque.append(
+            _SuiteEntry(data.name, data.source, "SUITE")
+        )
 
     def end_suite(self, data, result):
         self._stack_ctx_entries_deque.pop()
 
     def start_test(self, data, result):
         self._stack_ctx_entries_deque.append(
-            _TestEntry(data.name, data.source, data.lineno)
+            _TestEntry(data.name, data.source, data.lineno, "TEST")
         )
 
     def end_test(self, data, result):
         self._stack_ctx_entries_deque.pop()
 
     def log_message(self, message):
+        from robotframework_debug_adapter.message_utils import (
+            extract_source_and_line_from_message,
+        )
+
         # When debugging show any message in the console (if possible with the
         # current keyword as the source).
-        source = None
-        lineno = None
-        if self._stack_ctx_entries_deque:
-            lineno = 0
-            step_entry: _StepEntry = self._stack_ctx_entries_deque[-1]
-            source = step_entry.source
-            source = Source(path=source)
-            try:
-                lineno = step_entry.lineno
-            except AttributeError:
-                pass
-        self.write_message(
-            OutputEvent(
-                body=OutputEventBody(
-                    source=source,
-                    line=lineno,
-                    output=f"{message.message}\n",
-                    category="console",
+        try:
+            source = None
+            lineno = None
+
+            source_and_line = extract_source_and_line_from_message(message.message)
+
+            if source_and_line is not None:
+                source, lineno = source_and_line
+                source = Source(path=source)
+            else:
+                if self._stack_ctx_entries_deque:
+                    lineno = 0
+                    step_entry: _StepEntry = self._stack_ctx_entries_deque[-1]
+                    source = step_entry.source
+                    source = Source(path=source)
+                    try:
+                        lineno = step_entry.lineno
+                    except AttributeError:
+                        pass
+
+            self.write_message(
+                OutputEvent(
+                    body=OutputEventBody(
+                        source=source,
+                        line=lineno,
+                        output=f"{message.message}\n",
+                        category="console",
+                    )
                 )
             )
-        )
+            self._break_on_log_or_system_message(message)
+        except:
+            log.exception("Error handling log_message.")
+
+    def message(self, message):
+        if message.level in ("FAIL", "ERROR"):
+            # We also want to show in the console, not just check breaks.
+            return self.log_message(message)
+        else:
+            try:
+                self._break_on_log_or_system_message(message)
+            except:
+                log.exception("Error handling (system) message.")
+
+    def _break_on_log_or_system_message(self, message):
+        stop_reason = None
+        if message.level == "FAIL":
+            if self.break_on_log_failure:
+                stop_reason = ReasonEnum.REASON_EXCEPTION
+                exc_name = "Suspended due to logged failure: "
+
+        elif message.level == "ERROR":
+            if self.break_on_log_error:
+                stop_reason = ReasonEnum.REASON_EXCEPTION
+                exc_name = "Suspended due to logged error: "
+
+        if stop_reason is not None:
+            from robotframework_debug_adapter.message_utils import (
+                extract_source_and_line_from_message,
+            )
+
+            source_and_line = extract_source_and_line_from_message(message.message)
+            entry = None
+            if source_and_line is not None:
+                source, lineno = source_and_line
+                entry = _LogEntry(message.level, source, lineno, "LOG")
+                self._stack_ctx_entries_deque.append(entry)
+
+            self._exc_name = exc_name + message.message
+            self._exc_description = message.message
+            try:
+                self.wait_suspended(stop_reason)
+            finally:
+                self._exc_name = None
+                self._exc_description = None
+
+                if entry is not None:
+                    self._stack_ctx_entries_deque.pop()
 
 
 def _patch(
@@ -1111,6 +1244,7 @@ def install_robot_debugger() -> IRobotDebugger:
         DebugListener.on_end_test.register(impl.end_test)
 
         DebugListener.on_log_message.register(impl.log_message)
+        DebugListener.on_message.register(impl.message)
 
         # On RobotFramework 3.x and earlier 4.x dev versions, we do some monkey-patching because
         # the listener was not able to give linenumbers.
