@@ -10,12 +10,39 @@ from robotframework_ls.impl.protocols import (
     IKeywordCollector,
 )
 from robotframework_ls.impl.robot_specbuilder import KeywordArg
-from typing import Tuple, Sequence, List, Set, Dict
+from typing import Tuple, Sequence, List, Dict, Optional
 
 
 log = get_logger(__name__)
 
 
+def _build_docs(keyword_name, args, docs, docs_format):
+
+    if docs_format == "markdown":
+        # Multi-line approach (it's a bit too big -- maybe as an option?)
+        # if docs_format == "markdown":
+        #     arg_docs = "  \n&nbsp;&nbsp;&nbsp;&nbsp;".join(
+        #         ("**" + (x.replace("*", "\\*") + "**") for x in args)
+        #     )
+        #     return f"**{keyword_name}**  \n&nbsp;&nbsp;&nbsp;&nbsp;{arg_docs}\n\n{docs}"
+
+        if args:
+            escaped_args = (x.replace("*", "\\*") for x in args)
+            arg_docs = f'({", ".join(escaped_args)})'
+
+        else:
+            arg_docs = ""
+        return f"**{keyword_name}{arg_docs}**\n\n{docs}"
+    else:
+        if args:
+            arg_docs = f'({", ".join(args)})'
+
+        else:
+            arg_docs = ""
+
+        return f"{keyword_name}{arg_docs}\n\n{docs}"
+
+
 class _KeywordFoundFromAst(object):
 
     __slots__ = [
@@ -25,6 +52,7 @@ class _KeywordFoundFromAst(object):
         "_keyword_args",
         "completion_context",
         "completion_item_kind",
+        "_name_token",
         "__instance_cache__",
     ]
 
@@ -37,9 +65,13 @@ class _KeywordFoundFromAst(object):
         completion_context,
         completion_item_kind,
     ):
+        from robot.api import Token
+
         self._module_ast = module_ast
         self._keyword_node = keyword_node
 
+        self._name_token = keyword_node.header.get_token(Token.KEYWORD_NAME)
+
         self._keyword_name = keyword_name
         self._keyword_args = keyword_args
         self.completion_context = completion_context
@@ -79,7 +111,8 @@ class _KeywordFoundFromAst(object):
 
         docs = ast_utils.get_documentation(self._keyword_node)
         args = [x.original_arg for x in self.keyword_args]
-        return "%s(%s)\n\n%s" % (self.keyword_name, ", ".join(args), docs)
+
+        return _build_docs(self.keyword_name, args, docs, "markdown")
 
     @property
     @instance_cache
@@ -90,18 +123,34 @@ class _KeywordFoundFromAst(object):
 
     @property
     def lineno(self):
-        return self._keyword_node.lineno - 1
+        return self._name_token.lineno - 1
 
     @property
     def end_lineno(self):
-        return self._keyword_node.end_lineno - 1
+        return self._name_token.lineno - 1
 
     @property
     def col_offset(self):
-        return self._keyword_node.col_offset
+        return self._name_token.col_offset
 
     @property
     def end_col_offset(self):
+        return self._name_token.end_col_offset
+
+    @property
+    def scope_lineno(self) -> Optional[int]:
+        return self._keyword_node.lineno - 1
+
+    @property
+    def scope_end_lineno(self) -> Optional[int]:
+        return self._keyword_node.end_lineno - 1
+
+    @property
+    def scope_col_offset(self) -> Optional[int]:
+        return self._keyword_node.col_offset
+
+    @property
+    def scope_end_col_offset(self) -> Optional[int]:
         return self._keyword_node.end_col_offset
 
     def __typecheckself__(self) -> None:
@@ -184,6 +233,22 @@ class _KeywordFoundFromLibrary(object):
     def end_col_offset(self):
         return 0
 
+    @property
+    def scope_lineno(self) -> Optional[int]:
+        return self.lineno
+
+    @property
+    def scope_end_lineno(self) -> Optional[int]:
+        return self.end_lineno
+
+    @property
+    def scope_col_offset(self) -> Optional[int]:
+        return self.col_offset
+
+    @property
+    def scope_end_col_offset(self) -> Optional[int]:
+        return self.end_col_offset
+
     @property
     @instance_cache
     def _docs_and_format(self) -> Tuple[str, str]:
@@ -192,7 +257,8 @@ class _KeywordFoundFromLibrary(object):
         docs, docs_format = docs_and_format(self._keyword_doc)
         if self.keyword_args:
             args = [x.original_arg for x in self.keyword_args]
-            docs = "%s(%s)\n\n%s" % (self.keyword_name, ", ".join(args), docs)
+
+            docs = _build_docs(self.keyword_name, args, docs, docs_format)
 
         return docs, docs_format
 
@@ -218,6 +284,8 @@ def _collect_completions_from_ast(
     from robotframework_ls.impl import ast_utils
     from robocorp_ls_core.lsp import CompletionItemKind
 
+    found = {}
+
     for keyword in ast_utils.iter_keywords(ast):
         completion_context.check_cancelled()
         keyword_name = keyword.node.name
@@ -226,17 +294,21 @@ def _collect_completions_from_ast(
             for arg in ast_utils.iter_keyword_arguments_as_str(keyword.node):
                 keyword_args.append(KeywordArg(arg))
 
-            collector.on_keyword(
-                _KeywordFoundFromAst(
-                    ast,
-                    keyword.node,
-                    keyword_name,
-                    keyword_args,
-                    completion_context,
-                    CompletionItemKind.Function,
-                )
+            found[keyword_name] = _KeywordFoundFromAst(
+                ast,
+                keyword.node,
+                keyword_name,
+                keyword_args,
+                completion_context,
+                CompletionItemKind.Function,
             )
 
+    # We notify afterwards because if multiple definitions of the same
+    # keyword are found, we just want to report the last one (as is the
+    # case for the interactive console).
+    for keyword_found in found.values():
+        collector.on_keyword(keyword_found)
+
 
 def _collect_current_doc_keywords(
     completion_context: ICompletionContext, collector: IKeywordCollector
@@ -334,6 +406,7 @@ def _collect_libraries_keywords(
             node_name_tok = node.get_token(Token.NAME)
             if node_name_tok is not None:
                 collector.on_unresolved_library(
+                    completion_context,
                     node.name,
                     node_name_tok.lineno,
                     node_name_tok.lineno,
@@ -342,6 +415,7 @@ def _collect_libraries_keywords(
                 )
             else:
                 collector.on_unresolved_library(
+                    completion_context,
                     library_info.name,
                     node.lineno,
                     node.end_lineno,
@@ -361,6 +435,7 @@ def _collect_resource_imports_keywords(
             node_name_tok = node.get_token(Token.NAME)
             if node_name_tok is not None:
                 collector.on_unresolved_resource(
+                    completion_context,
                     node.name,
                     node_name_tok.lineno,
                     node_name_tok.lineno,
@@ -369,6 +444,7 @@ def _collect_resource_imports_keywords(
                 )
             else:
                 collector.on_unresolved_resource(
+                    completion_context,
                     node.name,
                     node.lineno,
                     node.end_lineno,
@@ -409,6 +485,7 @@ class _CollectKeywordNameToKeywordFound:
 
     def on_unresolved_library(
         self,
+        completion_context: ICompletionContext,
         library_name: str,
         lineno: int,
         end_lineno: int,
@@ -419,6 +496,7 @@ class _CollectKeywordNameToKeywordFound:
 
     def on_unresolved_resource(
         self,
+        completion_context: ICompletionContext,
         resource_name: str,
         lineno: int,
         end_lineno: int,
