@@ -219,6 +219,7 @@ TIMEOUT = "TIMEOUT"
 TAGS = "TAGS"
 ARGUMENTS = "ARGUMENTS"
 RETURN = "RETURN"
+RETURN_SETTING = RETURN
 
 NAME = "NAME"
 VARIABLE = "VARIABLE"
@@ -230,8 +231,17 @@ FOR = "FOR"
 FOR_SEPARATOR = "FOR SEPARATOR"
 END = "END"
 IF = "IF"
+INLINE_IF = "INLINE IF"
 ELSE_IF = "ELSE IF"
 ELSE = "ELSE"
+TRY = "TRY"
+EXCEPT = "EXCEPT"
+FINALLY = "FINALLY"
+AS = "AS"
+WHILE = "WHILE"
+RETURN_STATEMENT = "RETURN STATEMENT"
+CONTINUE = "CONTINUE"
+BREAK = "BREAK"
 
 SEPARATOR = "SEPARATOR"
 COMMENT = "COMMENT"
@@ -271,7 +281,24 @@ HEADER_TOKENS = frozenset(
     (SETTING_HEADER, VARIABLE_HEADER, TESTCASE_HEADER, KEYWORD_HEADER, COMMENT_HEADER)
 )
 ALLOW_VARIABLES = frozenset((NAME, ARGUMENT, TESTCASE_NAME, KEYWORD_NAME))
-
 CONTROL_TOKENS = frozenset(
-    (FOR, IF, ELSE, ELSE_IF, END, WITH_NAME, FOR_SEPARATOR, ASSIGN)
+    (
+        FOR,
+        IF,
+        INLINE_IF,
+        ELSE,
+        ELSE_IF,
+        END,
+        WITH_NAME,
+        FOR_SEPARATOR,
+        ASSIGN,
+        TRY,
+        EXCEPT,
+        FINALLY,
+        AS,
+        WHILE,
+        RETURN_STATEMENT,
+        CONTINUE,
+        BREAK,
+    )
 )
