@@ -1,8 +1,11 @@
 from robotframework_ls.impl.protocols import ICompletionContext, IKeywordFound
-from typing import List, Optional
+from typing import List, Optional, Union
 
 
 def signature_help(completion_context: ICompletionContext) -> Optional[dict]:
+    from robocorp_ls_core.lsp import MarkupContent
+    from robocorp_ls_core.lsp import MarkupKind
+
     keyword_definition = completion_context.get_current_keyword_definition()
     if keyword_definition is not None:
 
@@ -16,7 +19,14 @@ def signature_help(completion_context: ICompletionContext) -> Optional[dict]:
         lst = [arg.original_arg for arg in keyword_args]
 
         label = "%s(%s)" % (keyword_found.keyword_name, ", ".join(lst))
-        documentation = keyword_found.docs
+        docs_format = keyword_found.docs_format
+
+        documentation: Union[str, MarkupContent]
+        if docs_format == "markdown":
+            documentation = MarkupContent(MarkupKind.Markdown, keyword_found.docs)
+        else:
+            documentation = keyword_found.docs
+
         parameters: List[ParameterInformation] = [
             # Note: the label here is to highlight a part of the main signature label!
             # (let's leave this out for now)
