@@ -31,6 +31,22 @@ class RobotStringMatcher(object):
 
         return False
 
+    def is_same_variable_name(self, variable_name):
+        from robotframework_ls.impl.text_utilities import is_variable_text
+
+        try:
+            filter_var_name = self._filter_var_name
+        except AttributeError:
+            if is_variable_text(self.filter_text):
+                filter_var_name = self._filter_var_name = self.filter_text[2:-1]
+            else:
+                filter_var_name = self._filter_var_name = self.filter_text
+
+        if is_variable_text(variable_name):
+            variable_name = variable_name[2:-1]
+
+        return filter_var_name == normalize_robot_name(variable_name)
+
 
 class MatcherWithResourceOrLibraryName(RobotStringMatcher):
     def __init__(self, resource_or_library_name, qualifier):
@@ -42,6 +58,9 @@ class MatcherWithResourceOrLibraryName(RobotStringMatcher):
         """
         RobotStringMatcher.__init__(self, qualifier)
         self.resource_or_library_name = resource_or_library_name
+        self.resource_or_library_name_normalized = normalize_robot_name(
+            resource_or_library_name
+        )
 
     def accepts_keyword(self, keyword_found):
         """
@@ -51,7 +70,7 @@ class MatcherWithResourceOrLibraryName(RobotStringMatcher):
         if name is None:
             name = keyword_found.resource_name or keyword_found.library_name
 
-        if name == self.resource_or_library_name:
+        if normalize_robot_name(name) == self.resource_or_library_name_normalized:
             return self.accepts_keyword_name(keyword_found.keyword_name)
         return False
 
@@ -60,7 +79,7 @@ class MatcherWithResourceOrLibraryName(RobotStringMatcher):
         if name is None:
             name = keyword_found.resource_name or keyword_found.library_name
 
-        if name == self.resource_or_library_name:
+        if normalize_robot_name(name) == self.resource_or_library_name_normalized:
             return self.is_keyword_name_match(keyword_found.keyword_name)
         return False
 
