@@ -285,6 +285,16 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
             self._build_msg("hover", doc_uri=doc_uri, line=line, col=col)
         )
 
+    def request_document_highlight(
+        self, doc_uri: str, line: int, col: int
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+        return self.request_async(
+            self._build_msg("document_highlight", doc_uri=doc_uri, line=line, col=col)
+        )
+
     def request_references(
         self, doc_uri: str, line: int, col: int, include_declaration: bool
     ) -> Optional[IIdMessageMatcher]:
