@@ -217,7 +217,8 @@ class CompletionItem(_Base):
 
 
 class MarkupContent(_Base):
-    def __init__(self, kind: "MarkupKind", value: str):
+    def __init__(self, kind: str, value: str):
+        # kind is one of the MarkupKind constants.
         self.kind = kind
         self.value = value
 
@@ -337,10 +338,10 @@ class TextDocumentContentChangeEvent(_Base):
 
 class LocationLink(_Base):
     def __init__(
-        self, original_selection_range, target_uri, target_range, target_selection_range
+        self, origin_selection_range, target_uri, target_range, target_selection_range
     ):
         """
-        :param original_selection_range:
+        :param origin_selection_range:
             Span of the origin of this link.
             Used as the underlined span for mouse interaction. Defaults to the word range at
             the mouse position.
@@ -357,7 +358,7 @@ class LocationLink(_Base):
             The range that should be selected and revealed when this link is being followed, e.g the name of a function.
             Must be contained by the the `targetRange`. See also `DocumentSymbol#range`
         """
-        self.originalSelectionRange = original_selection_range
+        self.originSelectionRange = origin_selection_range
         self.targetUri = target_uri
         self.targetRange = target_range
         self.targetSelectionRange = target_selection_range
@@ -419,6 +420,15 @@ class TextDocumentTypedDict(TypedDict, total=False):
     text: Optional[str]
 
 
+class TextDocumentIdentifierTypedDict(TypedDict):
+    uri: str
+
+
+class TextDocumentPositionParamsTypedDict(TypedDict, total=False):
+    textDocument: TextDocumentIdentifierTypedDict
+    position: PositionTypedDict
+
+
 class MarkupContentTypedDict(TypedDict):
     kind: str  # "plaintext" | "markdown"
     value: str
@@ -540,6 +550,11 @@ class HoverTypedDict(TypedDict, total=False):
     range: RangeTypedDict  # Optional
 
 
+class DocumentHighlightTypedDict(TypedDict, total=False):
+    range: RangeTypedDict
+    kind: int  # Optional
+
+
 class ResponseErrorTypedDict(TypedDict, total=False):
     code: int
     message: str
@@ -558,6 +573,12 @@ class HoverResponseTypedDict(TypedDict, total=False):
     error: ResponseErrorTypedDict  # Optional
 
 
+class DocumentHighlightResponseTypedDict(TypedDict, total=False):
+    id: Union[int, str, None]
+    result: DocumentHighlightTypedDict  # Optional
+    error: ResponseErrorTypedDict  # Optional
+
+
 class ReferencesResponseTypedDict(TypedDict, total=False):
     id: Union[int, str, None]
     result: List[LocationTypedDict]  # Optional
